@@ -2,7 +2,6 @@
 Contains several unrelated utility functions used across the SDK.
 """
 
-import json
 import platform
 import re
 from pathlib import Path
@@ -21,13 +20,22 @@ from typing import (
 
 import deprecation
 import numpy as np
-from requests import Response
+import orjson as json
+import requests
+from jsonschema import exceptions, validators
+from requests import Response, request
 from rich.progress import ProgressType, track
 from upolygon import draw_polygon
 
 import darwin.datatypes as dt
 from darwin.config import Config
-from darwin.exceptions import OutdatedDarwinJSONFormat, UnsupportedFileType
+from darwin.exceptions import (
+    AnnotationFileValidationError,
+    MissingSchema,
+    OutdatedDarwinJSONFormat,
+    UnknownAnnotationFileSchema,
+    UnsupportedFileType,
+)
 from darwin.version import __version__
 
 if TYPE_CHECKING:
@@ -48,11 +56,33 @@ SUPPORTED_VIDEO_EXTENSIONS = [
 ]
 SUPPORTED_EXTENSIONS = SUPPORTED_IMAGE_EXTENSIONS + SUPPORTED_VIDEO_EXTENSIONS
 
+_darwin_schema_cache = {}
 
-def is_extension_allowed(extension: str) -> bool:
+
+def is_extension_allowed_by_filename(filename: str) -> bool:
     """
     Returns whether or not the given video or image extension is allowed.
 
+    Parameters
+    ----------
+    filename : str
+        The filename.
+
+    Returns
+    -------
+    bool
+        Whether or not the given extension of the filename is allowed.
+    """
+    return any([filename.lower().endswith(ext) for ext in SUPPORTED_EXTENSIONS])
+
+
+@deprecation.deprecated(deprecated_in="0.8.4", current_version=__version__)
+def is_extension_allowed(extension: str) -> bool:
+    """
+    Returns whether or not the given extension is allowed.
+    @Deprecated. Use is_extension_allowed_by_filename instead, and pass full filename.
+    This is due to the fact that some extensions now include multiple dots, e.g. .nii.gz
+
     Parameters
     ----------
     extension : str
@@ -66,6 +96,24 @@ def is_extension_allowed(extension: str) -> bool:
     return extension.lower() in SUPPORTED_EXTENSIONS
 
 
+def is_image_extension_allowed_by_filename(filename: str) -> bool:
+    """
+    Returns whether or not the given image extension is allowed.
+
+    Parameters
+    ----------
+    filename : str
+        The image extension.
+
+    Returns
+    -------
+    bool
+        Whether or not the given extension is allowed.
+    """
+    return any([filename.lower().endswith(ext) for ext in SUPPORTED_IMAGE_EXTENSIONS])
+
+
+@deprecation.deprecated(deprecated_in="0.8.4", current_version=__version__)
 def is_image_extension_allowed(extension: str) -> bool:
     """
     Returns whether or not the given image extension is allowed.
@@ -83,6 +131,24 @@ def is_image_extension_allowed(extension: str) -> bool:
     return extension.lower() in SUPPORTED_IMAGE_EXTENSIONS
 
 
+def is_video_extension_allowed_by_filename(extension: str) -> bool:
+    """
+    Returns whether or not the given image extension is allowed.
+
+    Parameters
+    ----------
+    extension : str
+        The image extension.
+
+    Returns
+    -------
+    bool
+        Whether or not the given extension is allowed.
+    """
+    return any([extension.lower().endswith(ext) for ext in SUPPORTED_VIDEO_EXTENSIONS])
+
+
+@deprecation.deprecated(deprecated_in="0.8.4", current_version=__version__)
 def is_video_extension_allowed(extension: str) -> bool:
     """
     Returns whether or not the given video extension is allowed.
@@ -209,14 +275,22 @@ def find_files(
 
     for f in files:
         path = Path(f)
-        if path.is_dir():
-            found_files.extend([f for f in path.glob(pattern) if is_extension_allowed("".join(f.suffixes))])
-        elif is_extension_allowed("".join(path.suffixes)):
+        if path.is_dir() == True:
+            found_files.extend(
+                [
+                    path_object
+                    for path_object in path.glob(pattern)
+                    if is_extension_allowed_by_filename(str(path_object))
+                ]
+            )
+        elif is_extension_allowed_by_filename(str(path)):
             found_files.append(path)
         else:
             raise UnsupportedFileType(path)
 
-    return [f for f in found_files if f not in map(Path, files_to_exclude)]
+    files_to_exclude_full_paths = [str(Path(f)) for f in files_to_exclude]
+
+    return [f for f in found_files if str(f) not in files_to_exclude_full_paths]
 
 
 def secure_continue_request() -> bool:
@@ -273,6 +347,43 @@ def _get_local_filename(metadata: Dict[str, Any]) -> str:
         return metadata["filename"]
 
 
+def _get_schema(data: dict) -> Optional[dict]:
+    version = _parse_version(data)
+    schema_url = data.get("schema_ref") or _default_schema(version)
+    if not schema_url:
+        return None
+    if schema_url not in _darwin_schema_cache:
+        response = requests.get(schema_url)
+        response.raise_for_status()
+        schema = response.json()
+        _darwin_schema_cache[schema_url] = schema
+    return _darwin_schema_cache[schema_url]
+
+
+def validate_file_against_schema(path: Path) -> List:
+    data, _ = load_data_from_file(path)
+    return validate_data_against_schema(data)
+
+
+def validate_data_against_schema(data) -> List:
+    try:
+        schema = _get_schema(data)
+    except requests.exceptions.RequestException as e:
+        raise MissingSchema(f"Error retrieving schema from url: {e}")
+    if not schema:
+        raise MissingSchema("Schema not found")
+    validator = validators.Draft202012Validator(schema)
+    errors = list(validator.iter_errors(data))
+    return errors
+
+
+def load_data_from_file(path: Path):
+    with path.open() as infile:
+        data = json.loads(infile.read())
+    version = _parse_version(data)
+    return data, version
+
+
 def parse_darwin_json(path: Path, count: Optional[int]) -> Optional[dt.AnnotationFile]:
     """
     Parses the given JSON file in v7's darwin proprietary format. Works for images, split frame
@@ -299,18 +410,18 @@ def parse_darwin_json(path: Path, count: Optional[int]) -> Optional[dt.Annotatio
     """
 
     path = Path(path)
-    with path.open() as f:
-        data = json.load(f)
-        if "annotations" not in data:
-            return None
 
-        if _parse_version(data).major == 2:
-            return _parse_darwin_v2(path, data)
+    data, version = load_data_from_file(path)
+    if "annotations" not in data:
+        return None
+
+    if version.major == 2:
+        return _parse_darwin_v2(path, data)
+    else:
+        if "fps" in data["image"] or "frame_count" in data["image"]:
+            return _parse_darwin_video(path, data, count)
         else:
-            if "fps" in data["image"] or "frame_count" in data["image"]:
-                return _parse_darwin_video(path, data, count)
-            else:
-                return _parse_darwin_image(path, data, count)
+            return _parse_darwin_image(path, data, count)
 
 
 def _parse_darwin_v2(path: Path, data: Dict[str, Any]) -> dt.AnnotationFile:
@@ -919,7 +1030,7 @@ def get_response_content(response: Response) -> Any:
 
 def _parse_version(data) -> dt.AnnotationFileVersion:
     version_string = data.get("version", "1.0")
-    major, minor, suffix = re.findall("^(\d+)\.(\d+)(.*)$", version_string)[0]
+    major, minor, suffix = re.findall(r"^(\d+)\.(\d+)(.*)$", version_string)[0]
     return dt.AnnotationFileVersion(int(major), int(minor), suffix)
 
 
@@ -931,3 +1042,11 @@ def _data_to_annotations(data: Dict[str, Any]) -> List[Union[dt.Annotation, dt.V
         filter(None, map(_parse_darwin_video_annotation, raw_video_annotations))
     )
     return [*image_annotations, *video_annotations]
+
+
+def _supported_schema_versions():
+    return {(2, 0, ""): "https://darwin-public.s3.eu-west-1.amazonaws.com/darwin_json/2.0/schema.json"}
+
+
+def _default_schema(version: dt.AnnotationFileVersion):
+    return _supported_schema_versions().get((version.major, version.minor, version.suffix))
